import typer
import httpx
import lxml.html
import sqlite3
from typing_extensions import Annotated
from .db import initialize_db, db
from .sync import full_sync
from .tui.things import run as things_tui
from .controller.things import add_thing

app = typer.Typer()


@app.command()
def new(
    name: Annotated[str, typer.Option("-n", "--name", help="name")] = "",
    category: Annotated[str, typer.Option("-c", "--category", help="name")] = "",
    due: Annotated[str, typer.Option("-d", "--due", help="due")] = "",
    url: Annotated[str, typer.Option("-u", "--url", help="URL")] = "",
):
    initialize_db()
    required = ["name", "category"]

    if url and not name:
        resp = httpx.get(url, follow_redirects=True)
        doc = lxml.html.fromstring(resp.text)
        try:
            name = doc.xpath("//title/text()")[0]
            print(f"fetched {url} name... {name}")
        except IndexError:
            name = ""

    if "name" in required and not name:
        name = typer.prompt("Name")
    if "category" in required and not category:
        category = typer.prompt("Category")
    # due = typer.prompt("Due (YYYY-MM-DD):")
    # TODO: validate/allow blank

    add_thing(name, category, due)
    typer.echo("Created new thing!")


@app.command()
def table(
    view: Annotated[str, typer.Option("-v", "--view", help="saved view")] = "tasks",
):
    initialize_db()
    things_tui(view)


@app.command()
def backup(backup_path: str):
    """
    Perform a SQLite backup using the .backup dot command
    """
    conn = db.connection()

    backup_conn = None
    try:
        backup_conn = sqlite3.connect(backup_path)
        conn.backup(backup_conn)
    finally:
        if backup_conn:
            backup_conn.close()


@app.command()
def sync():
    """
    Sync with tt server.
    """
    initialize_db()
    full_sync()


if __name__ == "__main__":
    app()
