import httpx
from collections import defaultdict
from .controller.things import get_things
from .config import get_view, get_column, SERVER_KEY, SERVER_URL


def sync_view(view_name):
    view = get_view(view_name)
    columns = {c: get_column(c) for c in view["columns"]}
    filters = view["filters"]
    resp = httpx.post(
        SERVER_URL + "view/",
        params={"api_key": SERVER_KEY},
        json={
            "name": view["name"],
            "columns": columns,
            "filters": filters,
            "sort": view["sort"],
        },
    )
    resp.raise_for_status()


def sync_thing(thing):
    resp = httpx.post(
        SERVER_URL + "thing/",
        params={"api_key": SERVER_KEY},
        json={
            "id": thing.id,
            "data": thing.data,
            "type": thing.type,
        },
    )
    resp.raise_for_status()
    return resp.json()


def full_sync():
    sync_view("tasks")
    actions = defaultdict(int)
    # all things for now
    for thing in get_things(None, None):
        action = sync_thing(thing)["action"]
        actions[action] += 1
    print(actions)
